"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * ApiClient "interface". Wraps the given apiClient as a singleton object.
 * Given apiClient must implement fetch().
 */
class ApiClient {
    constructor(apiClient, lang = "en", translations) {
        this.cache = {};
        this.on = {};
        this.flushCache = () => {
            this.cache = {};
        };
        this.apiClient = apiClient;
        this.lang = lang;
        this.translations = translations;
    }
    /**
     * Implementing apiClient must return a promise that passes the raw response as 1st arg.
     * @param path URL for GET.
     * @param query Object, where keys are param names and values are param values.
     * @returns a Promise.
     */
    fetchRaw(path, query, options) {
        const _query = Object.assign({ lang: this.lang }, (query || {}));
        if (!_query.lang) {
            delete _query.lang;
        }
        return this.apiClient.fetch(path, _query, options).catch(() => {
            throw new Error(this.translations[this.lang].RequestFailed);
        });
    }
    fetch(path, query, options = {}) {
        const { failSilently = false } = options, fetchOptions = __rest(options, ["failSilently"]);
        return this.fetchRaw(path, query, fetchOptions).then(response => {
            if (!failSilently && response.status >= 400) {
                throw new Error("Request failed");
            }
            return response.json();
        }).catch(() => {
            if (this.cache[path])
                delete this.cache[path][this.getCacheKey(query, options)];
            throw new Error(this.translations[this.lang].RequestFailed);
        });
    }
    getCacheKey(query, options) {
        return JSON.stringify(query) + JSON.stringify(options);
    }
    fetchCached(path, query, options) {
        const cacheKey = this.getCacheKey(query, options);
        if (!this.cache[path])
            this.cache[path] = {};
        this.cache[path][cacheKey] = cacheKey in this.cache[path] ? this.cache[path][cacheKey] : this.fetch(path, query, options);
        return this.cache[path][cacheKey];
    }
    invalidateCachePath(path) {
        delete this.cache[path];
        if (this.on[path]) {
            this.on[path].forEach(callback => callback());
        }
    }
    //TODO on invalidation callbacks
    invalidateCachePathQuery(path, query) {
        if (this.cache[path])
            delete this.cache[path][query];
    }
    onCachePathInvalidation(path, callback) {
        if (!this.on[path])
            this.on[path] = [];
        this.on[path].push(callback);
    }
    removeOnCachePathInvalidation(path, callback) {
        if (this.on[path]) {
            this.on[path] = this.on[path].filter(fn => fn !== callback);
        }
    }
    setLang(lang) {
        this.lang = lang;
        this.flushCache();
    }
}
exports.default = ApiClient;
